#include <avr/io.h>

;; Call-used registers: r18-27, 30-31 (Z)
;; Call-saved registers: r2-17, 28-29 (Y): frame pointer
#define	R18 18
#define	R19 19

;; used for main
#define	R2	2
#define	R3	3
#define	R4	4

#define	R20	20
#define	R21	21
#define	R22	22

;; used for interrupt
#define USART_IR_CNT	6
#define	R26 26
#define	R27 27

;;
#define R0 0

;; static

.section .text
; sine_table: .dc.b	1,2,3,4,5,6,7,8,9,10,

.section .bss
delta_temp:	.ds.b	6


.section .text

;; r20-21, 30: accumlator value
;; r2-4: delta 
.global DDSFunc
.func	DDSFunc
DDSFunc:
	clr		R2
	clr		R3
	clr		R4
	inc		R4
	
	clr		R20
	clr		R21
	ldi		R31, 0x07	; ZH
	clr		R30 		; ZL
	clr		USART_IR_CNT
	sei
	;; 9 clocks per loop
DDSFunc_sub:
	add		R20, R2
	adc		R21, R3
	adc		R30, R4		;ZL
	
	lpm		R22, Z		;3
	out		_SFR_IO_ADDR(PORTB), R22
;	out		_SFR_IO_ADDR(PORTB), R30	;debug
	
	rjmp	DDSFunc_sub	;2

	;; 8 clockes per loop
DDSFunc_HS_sub:
	add		R21, R3
	adc		R30, R4		;ZL
	
	lpm		R22, Z		;3
	out		_SFR_IO_ADDR(PORTB), R22
	
	rjmp	DDSFunc_HS_sub	;2

.endfunc

;; interrupt: overhead 3 clocks
.global USART_RX_vect
.func	USART_RX_vect
USART_RX_vect:
	in		R0, _SFR_IO_ADDR(SREG) ; Note [9]
;	in		R26, _SFR_IO_ADDR(PIND)
;	cpi		R26, 0x02		; check CSEL (PD1)
;	breq	USART_RX_INIT	; branch if CSEL is low
	
	ldi		YL, lo8(delta_temp)
	ldi		YH, hi8(delta_temp)
	
	in		R27, _SFR_IO_ADDR(UDR)
	cpi		R27, 0x7B
	brsh	USART_RX_INIT		; branch if >= (ex. '~')
	
USART_RX_1:
	;debug
;	andi	R27, 0x0F
;	mov		R4, R27
;	inc		USART_IR_CNT
;	out		_SFR_IO_ADDR(SREG), R0
;	reti
	;; store UDR to Y pointer
	add		YL, USART_IR_CNT
	adc		YH, R1	;; zero + c
	andi	R27, 0x0F
	st		Y, R27

	inc		USART_IR_CNT
	
	ldi		R26, 6
	cp		USART_IR_CNT, R26
	brge	USART_RX_2		; branch if >
	
	out		_SFR_IO_ADDR(SREG), R0
	reti
USART_RX_INIT:
	cpi		R27, 0xF0
	breq	USART_RX_RET_TO_LS
	cpi		R27, 0xF1
	breq	USART_RX_RET_TO_HS
	
	clr		USART_IR_CNT
	out		_SFR_IO_ADDR(SREG), R0
	reti

USART_RX_RET_TO_LS:
	pop		R27
	pop		R27
	sei
	rjmp	DDSFunc_sub
	
USART_RX_RET_TO_HS:
	pop		R27
	pop		R27
	sei
	rjmp	DDSFunc_HS_sub
	
	;debug
;	out		_SFR_IO_ADDR(SREG), R0
;	reti

USART_RX_2:

	;; if counter >= 6... clear counter and update accumlator
	clr		USART_IR_CNT
	
	ldi		YL, lo8(delta_temp)
	ldi		YH, hi8(delta_temp)
	
	ld		R2, Y+		; LSB
	ld		R26, Y+
	swap	R26
	or		R2, R26
	
	ld		R3, Y+
	ld		R26, Y+
	swap	R26
	or		R3, R26
	
	ld		R4, Y+
	ld		R26, Y+		; MSB
	swap	R26
	or		R4, R26
	
	out		_SFR_IO_ADDR(SREG), R0
	reti


.endfunc
